#pragma semicolon 1

#include <sourcemod>
#include <sdktools>
#include <morecolors>

#define PLUGIN_VERSION "1.2"

new Handle:h_GameName, String:s_GameName[32];
new Handle:h_Skin1, String:s_Skin1[PLATFORM_MAX_PATH];
new Handle:h_Skin2, String:s_Skin2[PLATFORM_MAX_PATH];
new Handle:h_Ranks, i_ranks; 

new bool:b_PlayerHasSkin[MAXPLAYERS+1];

new Handle:h_Database;

native HLStats_GetRank(client, Function:callback, any:data = 0);

public Plugin:myinfo =
{
	name = "[HLStatsX:CE] Rank skins",
	author = "FrozDark (HLModders LLC)",
	description = "HLStatsX:CE Plugin",
	version = PLUGIN_VERSION,
	url = "http://www.hlmod.ru/"
};

public APLRes:AskPluginLoad2(Handle:myself, bool:late, String:error[], err_max)
{	
	if (!SQL_CheckConfig("hlstats"))
	{
		strcopy(error, err_max, "Config \"hlstats\" is not present in the databases.cfg!");
		return APLRes_Failure;
	}
	CreateNative("HLStats_GetRank", Native_GetRank);
	
	return APLRes_Success;
}

public OnPluginStart() 
{
	CreateConVar("hlstats_skins_version", PLUGIN_VERSION, "HLStatsX:CE rank skins version", FCVAR_PLUGIN|FCVAR_SPONLY|FCVAR_REPLICATED|FCVAR_NOTIFY|FCVAR_CHEAT|FCVAR_DONTRECORD);
	
	h_Ranks = CreateConVar("hlstats_skins_ranks", "10", "How high to start HLStatsX:CE rank skins from. 0 to disable", 0, true, 0.0, true, 10.0);
	h_GameName = CreateConVar("hlstats_skins_game", "css", "The game servers' group name to use");
	h_Skin1 = CreateConVar("hlstats_skins_team_1", "", "Skin for the team index 2. <\"T\", \"RED\", \"USA\">");
	h_Skin2 = CreateConVar("hlstats_skins_team_2", "", "Skin for the team index 3. <\"CT\", \"BLU\", \"GER\">");
	
	i_ranks = GetConVarInt(h_Ranks);
	GetConVarString(h_GameName, s_GameName, sizeof(s_GameName));
	GetConVarString(h_Skin1, s_Skin1, sizeof(s_Skin1));
	GetConVarString(h_Skin2, s_Skin2, sizeof(s_Skin2));
	
	HookConVarChange(h_Ranks, ConVarChanges);
	HookConVarChange(h_GameName, ConVarChanges);
	HookConVarChange(h_Skin1, ConVarChanges);
	HookConVarChange(h_Skin2, ConVarChanges);
	
	HookEvent("player_spawn", Event_PlayerSpawn);
	HookEvent("player_team", Event_PlayerSpawn);
	HookEvent("round_start", Event_RoundStart);
	
	AutoExecConfig(true, "hlstats_skins");
	
	LoadTranslations("hlstats_skins");
	
	SQL_TConnect(OnDatabaseConnected, "hlstats");
}

public OnDatabaseConnected(Handle:owner, Handle:hndl, const String:error[], any:data)
{
	if (hndl == INVALID_HANDLE)
	{
		SetFailState("Couldn't connect to the database \"hlstats\"");
	}
	h_Database = hndl;
	PrintToServer("[HLStatsX:CE] Database connected");
	
	for (new client = 1; client <= MaxClients; client++)
	{
		if (IsClientInGame(client) && !IsFakeClient(client))
		{
			HLStats_GetRank(client, StatsCallback, client);
		}
	}
}

public Native_GetRank(Handle:plugin, numParams)
{
	new client = GetNativeCell(1);
	
	if (!(0 < client <= MaxClients) || !IsClientInGame(client))
	{
		ThrowNativeError(1, "Client index %d not in game or invalid", client);
		return false;
	}
	
	new Function:callback = GetNativeCell(2);
	new any:data = GetNativeCell(3);
	
	GetRank(client, Function:callback, any:data, Handle:plugin);
	
	return true;
}

GetRank(client, Function:callback, any:data, Handle:plugin)
{
	new Handle:pack = CreateDataPack();
	WritePackCell(pack, GetClientUserId(client));
	WritePackCell(pack, _:callback);
	WritePackCell(pack, data);
	WritePackCell(pack, _:plugin);
	
	decl String:auth[22];
	if (!GetClientAuthString(client, auth, sizeof(auth), false))
	{
		CreateTimer(2.0, ReupdateClient, pack);
		return;
	}
	
	decl String:query[1024];
	FormatEx(query, sizeof(query), "SELECT COUNT(*) AS rank FROM hlstats_Players WHERE hlstats_Players.game = '%s' AND hideranking = 0 AND skill > (SELECT skill from hlstats_Players JOIN hlstats_PlayerUniqueIds ON hlstats_Players.playerId = hlstats_PlayerUniqueIds.playerId WHERE uniqueID = MID('%s', 9) AND hlstats_PlayerUniqueIds.game = '%s') -1", s_GameName, auth, s_GameName);
	SQL_TQuery(h_Database, SQL_GetRankCallback, query, pack);
}

public Action:ReupdateClient(Handle:timer, any:dp)
{
	ResetPack(dp);
	new client = GetClientOfUserId(ReadPackCell(dp));
	if (!client)
	{
		CloseHandle(dp);
		return;
	}
	
	GetRank(client, Function:ReadPackCell(dp), any:ReadPackCell(dp), Handle:ReadPackCell(dp));
	CloseHandle(dp);
}

public SQL_GetRankCallback(Handle:owner, Handle:hndl, const String:error[], any:pack)
{
	ResetPack(pack);
	new userid = ReadPackCell(pack);
	new Function:callback = Function:ReadPackCell(pack);
	new any:args = ReadPackCell(pack);
	new Handle:plugin = Handle:ReadPackCell(pack);
	CloseHandle(pack);
	
	if (hndl == INVALID_HANDLE)
	{
		LogError("[HLStatsX:CE] Query Fail: %s", error);
		return;
	}
	if (SQL_FetchRow(hndl))
	{		
		CallRankCallback(userid, SQL_FetchInt(hndl, 0), callback, args, plugin);
	}
	else
	{
		CallRankCallback(userid, 0, callback, args, plugin);
	}
}

CallRankCallback(userid, rank, Function:callback, any:data, Handle:plugin)
{
	Call_StartFunction(plugin, callback);
	Call_PushCell(userid);
	Call_PushCell(rank);
	Call_PushCell(data);
	Call_Finish();
}

public OnMapStart()
{
	decl String:buffer[PLATFORM_MAX_PATH];
	BuildPath(Path_SM, buffer, sizeof(buffer), "configs/hlstats_dlist.ini");
	
	File_ReadDownloadList(buffer);
}

public OnConfigsExecuted()
{
	if (s_Skin1[0] && IsModelFile(s_Skin1))
	{
		PrecacheModel(s_Skin1, true);
	}
	else
	{
		s_Skin1[0] = '\0';
	}
	if (s_Skin2[0] && IsModelFile(s_Skin2))
	{
		PrecacheModel(s_Skin2, true);
	}
	else
	{
		s_Skin2[0] = '\0';
	}
}

public OnClientPutInServer(client)
{
	if (IsFakeClient(client) || i_ranks < 1)
	{
		return;
	}
	
	HLStats_GetRank(client, StatsCallback, client);
}

public OnClientDisconnect_Post(client)
{
	b_PlayerHasSkin[client] = false;
}

public StatsCallback(userid, rank, any:data)
{
	new client = GetClientOfUserId(userid);
	if (!client || !IsClientInGame(client))
	{
		return;
	}
	if (0 < rank <= i_ranks)
	{
		if (!b_PlayerHasSkin[client])
		{
			CPrintToChat(client, "%t", "InTop", rank, i_ranks);
		}
			
		b_PlayerHasSkin[client] = true;
		CreateTimer(0.1, SetModel, userid, TIMER_FLAG_NO_MAPCHANGE);
	}
	else if (b_PlayerHasSkin[client])
	{
		b_PlayerHasSkin[client] = false;
		CPrintToChat(client, "%t", "OutTop", i_ranks);
	}
}

public Action:SetModel(Handle:timer, any:userid)
{
	new client = GetClientOfUserId(userid);
	
	if (i_ranks < 1 || !client || !b_PlayerHasSkin[client] || !IsPlayerAlive(client))
	{
		return;
	}

	switch (GetClientTeam(client))
	{
		case 2 :
		{
			if (s_Skin1[0])
			{
				SetEntityModel(client, s_Skin1);
			}
		}
		case 3 :
		{
			if (s_Skin2[0])
			{
				SetEntityModel(client, s_Skin2);
			}
		}
	}
}

public Event_PlayerSpawn(Handle:event, const String:name[], bool:dontBroadcast)
{
	new userid = GetEventInt(event, "userid");
	CreateTimer(0.1, SetModel, userid, TIMER_FLAG_NO_MAPCHANGE);
}

public Event_RoundStart(Handle:event, const String:name[], bool:dontBroadcast)
{
	if (i_ranks < 1)
	{
		return;
	}
	for (new client = 1; client <= MaxClients; client++)
	{
		if (IsClientInGame(client) && !IsFakeClient(client))
		{
			HLStats_GetRank(client, StatsCallback, client);
		}
	}
}

// ======================================================================================================================================================

public ConVarChanges(Handle:convar, const String:oldValue[], const String:newValue[])
{
	if (convar == h_Ranks)
	{
		i_ranks = StringToInt(newValue);
	}
	else if (convar == h_GameName)
	{
		strcopy(s_GameName, sizeof(s_GameName), newValue);
	}
	else if (convar == h_Skin1)
	{
		strcopy(s_Skin1, sizeof(s_Skin1), newValue);
		if (s_Skin1[0] && IsModelFile(s_Skin1))
		{
			PrecacheModel(s_Skin1);
		}
		else
		{
			s_Skin1[0] = '\0';
		}
	}
	else if (convar == h_Skin2)
	{
		strcopy(s_Skin2, sizeof(s_Skin2), newValue);
		if (s_Skin2[0] && IsModelFile(s_Skin2))
		{
			PrecacheModel(s_Skin2);
		}
		else
		{
			s_Skin2[0] = '\0';
		}
	}
}

// ======================================================================================================================================================
new String:_smlib_empty_twodimstring_array[][] = { { '\0' } };
stock File_AddToDownloadsTable(const String:path[], bool:recursive=true, const String:ignoreExts[][]=_smlib_empty_twodimstring_array, size=0)
{
	if (path[0] == '\0') {
		return;
	}

	if (FileExists(path)) {
		
		new String:fileExtension[4];
		File_GetExtension(path, fileExtension, sizeof(fileExtension));
		
		if (StrEqual(fileExtension, "bz2", false) || StrEqual(fileExtension, "ztmp", false)) {
			return;
		}
		
		if (Array_FindString(ignoreExts, size, fileExtension) != -1) {
			return;
		}

		AddFileToDownloadsTable(path);
	}
	else if (recursive && DirExists(path)) {

		decl String:dirEntry[PLATFORM_MAX_PATH];
		new Handle:__dir = OpenDirectory(path);

		while (ReadDirEntry(__dir, dirEntry, sizeof(dirEntry))) {

			if (StrEqual(dirEntry, ".") || StrEqual(dirEntry, "..")) {
				continue;
			}
			
			Format(dirEntry, sizeof(dirEntry), "%s/%s", path, dirEntry);
			File_AddToDownloadsTable(dirEntry, recursive, ignoreExts, size);
		}
		
		CloseHandle(__dir);
	}
	else if (FindCharInString(path, '*', true)) {
		
		new String:fileExtension[4];
		File_GetExtension(path, fileExtension, sizeof(fileExtension));

		if (StrEqual(fileExtension, "*")) {

			decl
				String:dirName[PLATFORM_MAX_PATH],
				String:fileName[PLATFORM_MAX_PATH],
				String:dirEntry[PLATFORM_MAX_PATH];

			File_GetDirName(path, dirName, sizeof(dirName));
			File_GetFileName(path, fileName, sizeof(fileName));
			StrCat(fileName, sizeof(fileName), ".");

			new Handle:__dir = OpenDirectory(dirName);
			while (ReadDirEntry(__dir, dirEntry, sizeof(dirEntry))) {

				if (StrEqual(dirEntry, ".") || StrEqual(dirEntry, "..")) {
					continue;
				}

				if (strncmp(dirEntry, fileName, strlen(fileName)) == 0) {
					Format(dirEntry, sizeof(dirEntry), "%s/%s", dirName, dirEntry);
					File_AddToDownloadsTable(dirEntry, recursive, ignoreExts, size);
				}
			}

			CloseHandle(__dir);
		}
	}

	return;
}

stock File_ReadDownloadList(const String:path[])
{
	new Handle:file = OpenFile(path, "r");
	
	if (file  == INVALID_HANDLE) {
		return;
	}

	new String:buffer[PLATFORM_MAX_PATH];
	while (!IsEndOfFile(file)) {
		ReadFileLine(file, buffer, sizeof(buffer));
		
		new pos;
		pos = StrContains(buffer, "//");
		if (pos != -1) {
			buffer[pos] = '\0';
		}
		
		pos = StrContains(buffer, "#");
		if (pos != -1) {
			buffer[pos] = '\0';
		}

		pos = StrContains(buffer, ";");
		if (pos != -1) {
			buffer[pos] = '\0';
		}
		
		TrimString(buffer);
		
		if (buffer[0] == '\0') {
			continue;
		}

		File_AddToDownloadsTable(buffer);
	}

	CloseHandle(file);
}

stock File_GetExtension(const String:path[], String:buffer[], size)
{
	new extpos = FindCharInString(path, '.', true);
	
	if (extpos == -1)
	{
		buffer[0] = '\0';
		return;
	}

	strcopy(buffer, size, path[++extpos]);
}

stock Array_FindString(const String:array[][], size, const String:str[], bool:caseSensitive=true, start=0)
{
	if (start < 0) {
		start = 0;
	}

	for (new i=start; i < size; i++) {

		if (StrEqual(array[i], str, caseSensitive)) {
			return i;
		}
	}
	
	return -1;
}

stock bool:File_GetFileName(const String:path[], String:buffer[], size)
{	
	if (path[0] == '\0') {
		buffer[0] = '\0';
		return;
	}
	
	File_GetBaseName(path, buffer, size);
	
	new pos_ext = FindCharInString(buffer, '.', true);

	if (pos_ext != -1) {
		buffer[pos_ext] = '\0';
	}
}

stock bool:File_GetDirName(const String:path[], String:buffer[], size)
{	
	if (path[0] == '\0') {
		buffer[0] = '\0';
		return;
	}
	
	new pos_start = FindCharInString(path, '/', true);
	
	if (pos_start == -1) {
		pos_start = FindCharInString(path, '\\', true);
		
		if (pos_start == -1) {
			buffer[0] = '\0';
			return;
		}
	}
	
	strcopy(buffer, size, path);
	buffer[pos_start] = '\0';
}

stock bool:File_GetBaseName(const String:path[], String:buffer[], size)
{	
	if (path[0] == '\0') {
		buffer[0] = '\0';
		return;
	}
	
	new pos_start = FindCharInString(path, '/', true);
	
	if (pos_start == -1) {
		pos_start = FindCharInString(path, '\\', true);
	}
	
	pos_start++;
	
	strcopy(buffer, size, path[pos_start]);
}

bool:IsModelFile(const String:model[])
{
	decl String:buf[4];
	File_GetExtension(model, buf, sizeof(buf));
	if (!strcmp(buf, "mdl", false))
	{
		return true;
	}
	
	return false;
}
// ======================================================================================================================================================